import requests

def send_message(url: str, message: bytes):
  try:
    requests.post(url, data=message, headers={'Content-Type': 'application/octet-stream'})
  except requests.RequestException as e:
    print('ERROR: Failed to POST message to {}'.format(url))
    raise e



from typing import List
from secrets import token_bytes

from verity_sdk.utils import Context, get_message_type, get_problem_report_message_type, get_status_message_type
from verity_sdk.protocols.Protocol import Protocol


class QuestionAnswer(Protocol):
  MSG_FAMILY = 'questionanswer'
  MSG_FAMILY_VERSION = '1.0'

  # Messages
  ASK_QUESTION = 'ask-question'
  GET_STATUS = 'get-status'

  # Status
  QUESTION_SENT_STATUS = 0
  QUESTION_ANSWERED_STATUS = 1

  for_relationship: str
  question_text: str
  question_detail: str
  valid_responses: List[str]
  signature_required: bool


  def __init__(self,
               for_relationship: str,
               question_text: str,
               question_detail: str,
               valid_responses: List[str],
               signature_required: bool = True):
    self.for_relationship = for_relationship
    self.question_text = question_text
    self.question_detail = question_detail
    self.valid_responses = valid_responses
    self.signature_required = signature_required

    self.define_messages()

  def define_messages(self):
    self.messages = {
      self.ASK_QUESTION: {
        '@type': QuestionAnswer.get_message_type(self.ASK_QUESTION),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
        'text': self.question_text,
        'detail': self.question_detail,
        'valid_responses': self.valid_responses,
        'signature_required': self.signature_required,
        # '@timing': None,  # TODO add support for @timing
        # 'external_links': None  # TODO add support for external_links
      },
      self.GET_STATUS: {
        '@type': QuestionAnswer.get_message_type(self.GET_STATUS),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(QuestionAnswer.MSG_FAMILY, QuestionAnswer.MSG_FAMILY_VERSION, msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(QuestionAnswer.MSG_FAMILY, QuestionAnswer.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(QuestionAnswer.MSG_FAMILY, QuestionAnswer.MSG_FAMILY_VERSION)

  @staticmethod
  def get_random_bytes():
    NUM_BYTES = 32
    return token_bytes(NUM_BYTES)

  async def ask(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.ASK_QUESTION])

  async def status(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.GET_STATUS])



from verity_sdk.utils import Context, get_message_type, get_problem_report_message_type, get_status_message_type
from verity_sdk.protocols.Protocol import Protocol


class IssueCredential(Protocol):
  MSG_FAMILY = 'issue-credential'
  MSG_FAMILY_VERSION = '0.6'

  # Messages
  OFFER_CREDENTIAL = 'send-offer'
  ISSUE_CREDENTIAL = 'issue-credential'
  GET_STATUS = 'get-status'

  # Status
  OFFER_SENT_TO_USER_STATUS = 0
  OFFER_ACCEPTED_BY_USER_STATUS = 1
  CREDENTIAL_SENT_TO_USER_STATUS = 2

  for_relationship: str
  name: str
  cred_def_id: str
  credential_values: dict
  price: str

  def __init__(self, for_relationship: str, name: str, cred_def_id: str, credential_values: dict, price: str):
    self.for_relationship = for_relationship
    self.name = name
    self.cred_def_id = cred_def_id
    self.credential_values = credential_values
    self.price = price

    self.define_messages()

  def define_messages(self):
    self.messages = {
      self.OFFER_CREDENTIAL: {
        '@type': IssueCredential.get_message_type(self.OFFER_CREDENTIAL),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
        'name': self.name,
        'credDefId': self.cred_def_id,
        'credentialValues': self.credential_values,
        'price': self.price
      },
      self.ISSUE_CREDENTIAL: {
        '@type': IssueCredential.get_message_type(self.ISSUE_CREDENTIAL),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
      },
      self.GET_STATUS: {
        '@type': IssueCredential.get_message_type(self.GET_STATUS),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(IssueCredential.MSG_FAMILY, IssueCredential.MSG_FAMILY_VERSION, msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(IssueCredential.MSG_FAMILY, IssueCredential.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(IssueCredential.MSG_FAMILY, IssueCredential.MSG_FAMILY_VERSION)

  async def offer_credential(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.OFFER_CREDENTIAL])

  async def issue_credential(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.ISSUE_CREDENTIAL])

  async def status(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.GET_STATUS])



import pytest

from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context
from verity_sdk.protocols.WriteSchema import WriteSchema
from ..test_utils import get_test_config, send_stub, cleanup

schema_name = 'schema name'
schema_version = '0.1.0'
attrs = ('name', 'degree', 'age')


def test_init():
  write_schema = WriteSchema(schema_name, schema_version, attrs[0], attrs[1], attrs[2])

  assert write_schema.name == schema_name
  assert write_schema.version == schema_version
  assert len(write_schema.attrs) == 3
  assert write_schema.attrs == list(attrs)


@pytest.mark.asyncio
async def test_write():
  context = await Context.create(await get_test_config())
  write_schema = WriteSchema(schema_name, schema_version, attrs[0], attrs[1], attrs[2])
  write_schema.send = send_stub
  msg = await write_schema.write(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    WriteSchema.MSG_FAMILY,
    WriteSchema.MSG_FAMILY_VERSION,
    WriteSchema.WRITE_SCHEMA
  )
  assert msg['@id'] is not None
  assert msg['name'] == schema_name
  assert msg['version'] == schema_version
  assert msg['attrNames'] == list(attrs)

  await cleanup(context)


def test_get_message_type():
  writeSchema = WriteSchema(schema_name, schema_version, attrs[0], attrs[1], attrs[2])
  assert writeSchema.get_message_type('message_name') == '{};spec/{}/{}/message_name'.format(
    MESSAGE_TYPE_DID,
    WriteSchema.MSG_FAMILY,
    WriteSchema.MSG_FAMILY_VERSION
  )


def test_get_problem_report_message_type():
  writeSchema = WriteSchema(schema_name, schema_version, attrs[0], attrs[1], attrs[2])
  assert writeSchema.get_problem_report_message_type() == '{};spec/{}/{}/problem-report'.format(
    MESSAGE_TYPE_DID,
    WriteSchema.MSG_FAMILY,
    WriteSchema.MSG_FAMILY_VERSION
  )


def test_get_status_message_type():
  writeSchema = WriteSchema(schema_name, schema_version, attrs[0], attrs[1], attrs[2])
  assert writeSchema.get_status_message_type() == '{};spec/{}/{}/status'.format(
    MESSAGE_TYPE_DID,
    WriteSchema.MSG_FAMILY,
    WriteSchema.MSG_FAMILY_VERSION,
            )



import pytest

from verity_sdk.protocols.WriteCredentialDefinition import WriteCredentialDefinition
from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context
from test.test_utils import get_test_config, send_stub, cleanup

name = 'name'
schema_id = 'schema id'
tag = 'tag'
revocation_details = {'support_revocation': False}


def test_init():
  writeCredDef = WriteCredentialDefinition(name, schema_id, tag, revocation_details)

  assert writeCredDef.name == name
  assert writeCredDef.schema_id == schema_id
  assert writeCredDef.tag == tag
  assert writeCredDef.revocation_details == revocation_details
  assert len(writeCredDef.messages) == 1


@pytest.mark.asyncio
async def test_write():
  context = await Context.create(await get_test_config())
  writeCredDef = WriteCredentialDefinition(name, schema_id, tag, revocation_details)
  writeCredDef.send = send_stub
  msg = await writeCredDef.write(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    WriteCredentialDefinition.MSG_FAMILY,
    WriteCredentialDefinition.MSG_FAMILY_VERSION,
    WriteCredentialDefinition.WRITE_CRED_DEF
  )
  assert msg['@id'] is not None
  assert msg['name'] == name
  assert msg['schemaId'] == schema_id
  assert msg['tag'] == tag
  assert msg['revocationDetails'] == revocation_details

  await cleanup(context)



from verity_sdk.utils import Context, get_message_type, get_problem_report_message_type, get_status_message_type
from verity_sdk.protocols.Protocol import Protocol


class WriteCredentialDefinition(Protocol):
  MSG_FAMILY = 'write-cred-def'
  MSG_FAMILY_VERSION = '0.6'

  # Messages
  WRITE_CRED_DEF = 'write'

  # Status
  WRITE_SUCCESSFUL_STATUS = 0

  # Data members
  name: str
  schema_id: str
  tag: str
  revocation_details: dict

  def __init__(self, name: str, schema_id: str, tag: str = None, revocation_details: dict = None):
    self.name = name
    self.schema_id = schema_id
    self.tag = tag
    self.revocation_details = revocation_details
    self.define_messages()

  def define_messages(self):
    self.messages = {
      self.WRITE_CRED_DEF: {
        '@type': WriteCredentialDefinition.get_message_type(self.WRITE_CRED_DEF),
        '@id': self.get_new_id(),
        'name': self.name,
        'schemaId': self.schema_id,
        'tag': self.tag,
        'revocationDetails': self.revocation_details
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(WriteCredentialDefinition.MSG_FAMILY, WriteCredentialDefinition.MSG_FAMILY_VERSION,
                            msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(WriteCredentialDefinition.MSG_FAMILY,
                                           WriteCredentialDefinition.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(WriteCredentialDefinition.MSG_FAMILY, WriteCredentialDefinition.MSG_FAMILY_VERSION)

  async def write(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.WRITE_CRED_DEF])



import json
from uuid import uuid4 as uuid

import pytest
from indy import did, wallet

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import prepare_forward_message, pack_message_for_verity, \
  unpack_forward_message, get_message_type, get_problem_report_message_type, \
  MESSAGE_TYPE_DID, get_status_message_type
from verity_sdk.utils.Context import Context


async def get_test_config():
  test_config = {
    'walletName': str(uuid()),
    'walletKey': str(uuid()),
    'verityUrl': 'http://localhost:8080/agency/msg',
    'verityPublicVerkey': None,
    'verityPairwiseDID': None,
    'verityPairwiseVerkey': None,
    'sdkPairwiseVerkey': None,
    'endpointUrl': 'http://localhost:4000'
  }

  wallet_config = json.dumps({'id': test_config['walletName']})
  wallet_credentials = json.dumps({'key': test_config['walletKey']})
  await wallet.create_wallet(wallet_config, wallet_credentials)
  wallet_handle = await wallet.open_wallet(wallet_config, wallet_credentials)
  [_, their_public_verkey] = await did.create_and_store_my_did(wallet_handle, '{}')
  [their_did, their_verkey] = await did.create_and_store_my_did(wallet_handle, '{}')
  [_, my_verkey] = await did.create_and_store_my_did(wallet_handle, '{}')
  await wallet.close_wallet(wallet_handle)

  test_config['verityPublicVerkey'] = their_public_verkey
  test_config['verityPairwiseDID'] = their_did
  test_config['verityPairwiseVerkey'] = their_verkey
  test_config['sdkPairwiseVerkey'] = my_verkey

  return json.dumps(test_config)


async def cleanup(context: Context):
  if not context.wallet_closed:
    await context.close_wallet()
  await wallet.delete_wallet(context.wallet_config, context.wallet_credentials)


async def send_stub(context: Context, message: dict) -> bytes:
  return await Protocol.get_message(context, message)


@pytest.mark.asyncio
async def test_Context():
  test_config = await get_test_config()

  context = await Context.create(test_config)
  test_config = json.loads(test_config)
  assert context.wallet_name == test_config['walletName']
  assert context.wallet_key == test_config['walletKey']
  assert context.verity_url == test_config['verityUrl']
  assert context.verity_public_verkey == test_config['verityPublicVerkey']
  assert context.verity_pairwise_did == test_config['verityPairwiseDID']
  assert context.verity_pairwise_verkey == test_config['verityPairwiseVerkey']
  assert context.sdk_pairwise_verkey == test_config['sdkPairwiseVerkey']
  assert context.endpoint_url == test_config['endpointUrl']

  await cleanup(context)


def test_prepare_forward_message():
  test_did = '123'
  message = b'{"hello": "world"}'
  msg = prepare_forward_message(test_did, message)
  msg = json.loads(msg)

  assert msg['@type'] == 'did:sov:123456789abcdefghi1234;spec/routing/1.0/FWD'
  assert msg['@fwd'] == test_did
  assert msg['@msg'] == json.loads(message.decode('utf-8'))

@pytest.mark.asyncio
async def test_pack_message_for_verity_and_unpack_forward_message():
  message = {'hello': 'world'}
  context = await Context.create(await get_test_config())
  packed_message = await pack_message_for_verity(context, message)
  unpacked_message = await unpack_forward_message(context, packed_message)
  assert unpacked_message['hello'] == 'world'

  await cleanup(context)


def test_get_message_type():
  msg_type: str = 'did:sov:123456789abcdefghi1234;spec/credential/0.1/status'
  assert get_message_type('credential', '0.1', 'status') == msg_type


def test_get_problem_report_message_type():
  assert get_problem_report_message_type('credential', '0.1') == \
         '{};spec/credential/0.1/problem-report'.format(MESSAGE_TYPE_DID)


def test_get_status_message_type():
  assert get_status_message_type('credential', '0.1') ==\
         '{};spec/credential/0.1/status'.format(MESSAGE_TYPE_DID)



from typing import List

from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import Context, get_message_type, get_problem_report_message_type, get_status_message_type


class WriteSchema(Protocol):
  MSG_FAMILY = 'write-schema'
  MSG_FAMILY_VERSION = '0.6'

  # Messages
  WRITE_SCHEMA = 'write'

  # Status
  WRITE_SUCCESSFUL_STATUS = 0

  # Data members
  name: str
  version: str
  attrs: List[str]

  def __init__(self, name: str, version: str, *attrs: str):
    self.name = name
    self.version = version
    self.attrs = list(attrs)
    self.define_messages()

  def define_messages(self):
    self.messages = {
      self.WRITE_SCHEMA: {
        '@type': WriteSchema.get_message_type(self.WRITE_SCHEMA),
        '@id': self.get_new_id(),
        'name': self.name,
        'version': self.version,
        'attrNames': self.attrs
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(WriteSchema.MSG_FAMILY, WriteSchema.MSG_FAMILY_VERSION, msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(WriteSchema.MSG_FAMILY, WriteSchema.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(WriteSchema.MSG_FAMILY, WriteSchema.MSG_FAMILY_VERSION)

  async def write(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.WRITE_SCHEMA])



import json
import pytest

from verity_sdk.utils import unpack_forward_message
from verity_sdk.utils.Context import Context
from verity_sdk.protocols.Protocol import Protocol
from ..test_utils import get_test_config, cleanup


@pytest.mark.asyncio
async def test_get_message():
  message = {'hello': 'world'}
  context = await Context.create(await get_test_config())
  packed_message = await Protocol.get_message(context, message)
  unpacked_message = json.dumps(await unpack_forward_message(context, packed_message))
  assert json.dumps(message) == unpacked_message
  await cleanup(context)

@pytest.mark.asyncio
async def test_get_thread_block():
  protocol = Protocol()
  block = protocol.get_thread_block()
  assert block['thid'] is not None



import pytest

from verity_sdk.protocols.PresentProof import PresentProof
from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context
from test.test_utils import get_test_config, send_stub, cleanup

for_relationship = 'some_did'
name = 'Degree Verification'
proof_attrs = [
  {'name': 'name', 'restrictions': [{'issuer_did': 'did:sov:12345'}]},
  {'name': 'degree', 'restrictions': [{'issuer_did': 'did:sov:12345'}]}
]
proof_predicates = [
  {'name': 'age', 'p_type': 'GT', 'p_value': 18, 'restrictions': [{'issuer_did': 'did:sov:12345'}]}
]
revocation_interval = {'support_revocation': False}


def test_init():
  presentProof = PresentProof(for_relationship, name, proof_attrs, proof_predicates, revocation_interval)

  assert presentProof.for_relationship == for_relationship
  assert presentProof.name == name
  assert presentProof.proof_attrs == proof_attrs
  assert presentProof.proof_predicates == proof_predicates
  assert presentProof.revocation_interval == revocation_interval


@pytest.mark.asyncio
async def test_request():
  context = await Context.create(await get_test_config())
  presentProof = PresentProof(for_relationship, name, proof_attrs, proof_predicates, revocation_interval)
  presentProof.send = send_stub
  msg = await presentProof.request(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    PresentProof.MSG_FAMILY,
    PresentProof.MSG_FAMILY_VERSION,
    PresentProof.PROOF_REQUEST
  )
  assert msg['@id'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None
  assert msg['name'] == name
  assert msg['proofAttrs'] == proof_attrs
  assert msg['proofPredicates'] == proof_predicates
  assert msg['revocationInterval'] == revocation_interval

  await cleanup(context)

async def test_status():
  context = await Context.create(await get_test_config())
  presentProof = PresentProof(for_relationship, name, proof_attrs, proof_predicates, revocation_interval)
  presentProof.send = send_stub
  msg = await presentProof.status(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    PresentProof.MSG_FAMILY,
    PresentProof.MSG_FAMILY_VERSION,
    PresentProof.GET_STATUS
  )
  assert msg['@id'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None

  await cleanup(context)



from verity_sdk.utils import Context, get_message_type, get_problem_report_message_type, get_status_message_type
from verity_sdk.protocols.Protocol import Protocol


class Connecting(Protocol):
  MSG_FAMILY = 'connecting'
  MSG_FAMILY_VERSION = '0.6'

  # Messages
  CREATE_CONNECTION = 'CREATE_CONNECTION'

  # Status
  AWAITING_RESPONSE_STATUS = 0
  INVITE_ACCEPTED_STATUS = 1

  source_id: str
  phone_number: str
  include_public_did: bool

  def __init__(self, source_id: str, phone_number: str = None, include_public_did: bool = False):
    self.source_id = source_id
    self.phone_number = phone_number
    self.include_public_did = include_public_did

    self.define_messages()

  def define_messages(self):
    self.messages = {
      self.CREATE_CONNECTION: {
        '@type': Connecting.get_message_type(self.CREATE_CONNECTION),
        '@id': self.get_new_id(),
        'sourceId': self.source_id,
        'phoneNo': self.phone_number,
        'includePublicDID': self.include_public_did
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(Connecting.MSG_FAMILY, Connecting.MSG_FAMILY_VERSION, msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(Connecting.MSG_FAMILY, Connecting.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(Connecting.MSG_FAMILY, Connecting.MSG_FAMILY_VERSION)

  async def connect(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.CREATE_CONNECTION])



from typing import List
from verity_sdk.utils import Context, get_message_type, get_problem_report_message_type, get_status_message_type
from verity_sdk.protocols.Protocol import Protocol


class PresentProof(Protocol):
  MSG_FAMILY = 'present-proof'
  MSG_FAMILY_VERSION = '0.6'

  # Messages
  PROOF_REQUEST = 'request'
  GET_STATUS = 'get-status'

  # Status
  PROOF_REQUEST_SENT_STATUS = 0
  PROOF_RECEIVED_STATUS = 1

  for_relationship: str
  name: str
  proof_attrs: List[dict]
  proof_predicates: List[dict]
  revocation_interval: dict

  def __init__(self,
               for_relationship: str,
               name: str,
               proof_attrs: List[dict],
               proof_predicates: List[dict] = None,
               revocation_interval: dict = None):
    self.for_relationship = for_relationship
    self.name = name
    self.proof_attrs = proof_attrs
    self.proof_predicates = proof_predicates
    self.revocation_interval = revocation_interval or {}
    self.define_messages()

  def define_messages(self):
    self.messages = {
      self.PROOF_REQUEST: {
        '@type': PresentProof.get_message_type(self.PROOF_REQUEST),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
        'name': self.name,
        'proofAttrs': self.proof_attrs,
        'proofPredicates': self.proof_predicates,
        'revocationInterval': self.revocation_interval
      },
      self.GET_STATUS: {
        '@type': PresentProof.get_message_type(self.GET_STATUS),
        '@id': self.get_new_id(),
        '~for_relationship': self.for_relationship,
        '~thread': self.get_thread_block(),
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(PresentProof.MSG_FAMILY, PresentProof.MSG_FAMILY_VERSION, msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(PresentProof.MSG_FAMILY, PresentProof.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(PresentProof.MSG_FAMILY, PresentProof.MSG_FAMILY_VERSION)

  async def request(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.PROOF_REQUEST])

  async def status(self, context: Context) -> bytes:
    return await self.send(context, self.messages[self.GET_STATUS])



import pytest

from verity_sdk.protocols.Connecting import Connecting
from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context
from test.test_utils import get_test_config, send_stub, cleanup

source_id = '12345'
phone_number = '1234357890'
include_public_did = True


def test_init():
  connecting = Connecting(source_id, phone_number, include_public_did)

  assert connecting.source_id == source_id
  assert connecting.phone_number == phone_number
  assert connecting.include_public_did == include_public_did
  assert len(connecting.messages) == 1


@pytest.mark.asyncio
async def test_connect():
  context = await Context.create(await get_test_config())
  connecting = Connecting(source_id, phone_number, include_public_did)
  connecting.send = send_stub
  msg = await connecting.connect(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/connecting/0.6/CREATE_CONNECTION'.format(MESSAGE_TYPE_DID)
  assert msg['@id'] is not None
  assert msg['sourceId'] == source_id
  assert msg['phoneNo'] == phone_number
  assert msg['includePublicDID'] == include_public_did

  await cleanup(context)



import pytest

from verity_sdk.handlers import is_problem_report, Handlers, AddHandler


def test_is_problem_report():
  assert is_problem_report('vs.service/enroll/0.1/problem-report')
  assert not is_problem_report('vs.service/enroll/0.1/enroll')

#pylint: disable=unused-variable
@pytest.mark.asyncio
async def test_decorator():
  handlers = Handlers()

  @AddHandler(handlers, 'some type', 0)
  async def some_handler(msg: dict) -> None:
    print('do something with {}'.format(msg))

  @AddHandler(handlers, 'another type', 1)
  async def another_handler(msg: dict) -> None:
    print('do another thing with {}'.format(msg))

  assert len(handlers.handlers) == 2
  assert handlers.handlers[0].message_type == 'some type'
  assert handlers.handlers[0].message_status == 0
  await handlers.handlers[0]({})
  assert handlers.handlers[1].message_type == 'another type'
  assert handlers.handlers[1].message_status == 1
  await handlers.handlers[1]({})



import json

from indy import wallet

class Context:

  wallet_name: str
  wallet_key: str
  verity_url: str
  verity_public_verkey: str
  verity_pairwise_did: str
  verity_pairwise_verkey: str
  sdk_pairwise_verkey: str
  endpoint_url: str
  wallet_config: str
  wallet_credentials: str
  wallet_handle: int
  wallet_closed: bool

  @classmethod
  async def create(cls, config):
    context = cls()
    config = json.loads(config)

    context.wallet_name = config['walletName']
    context.wallet_key = config['walletKey']
    context.verity_url = config['verityUrl']
    context.verity_public_verkey = config['verityPublicVerkey']
    context.verity_pairwise_did = config['verityPairwiseDID']
    context.verity_pairwise_verkey = config['verityPairwiseVerkey']
    context.sdk_pairwise_verkey = config['sdkPairwiseVerkey']
    context.endpoint_url = config['endpointUrl']
    context.wallet_config = json.dumps({'id': config['walletName']})
    context.wallet_credentials = json.dumps({'key': config['walletKey']})
    context.wallet_handle = await wallet.open_wallet(context.wallet_config, context.wallet_credentials)
    context.wallet_closed = False

    return context

  async def close_wallet(self):
    await wallet.close_wallet(self.wallet_handle)
    self.wallet_closed = True



from typing import Callable, List, Optional, Coroutine, Any

from verity_sdk.utils import Context, unpack_message


def is_problem_report(message_type: str) -> bool:
  return message_type.split('/')[3] == 'problem-report'


class Handler():
  message_type: str
  message_status: Optional[int]
  message_handler: Callable[[dict], Coroutine[Any, Any, None]]

  def __init__(
      self,
      message_type: str,
      message_status: Optional[int],
      message_handler: Callable[[dict], Coroutine[Any, Any, None]]):
    self.message_type = message_type
    self.message_status = message_status
    self.message_handler = message_handler

  def handles(self, message: dict) -> bool:
    if self.message_status is not None:
      return message['@type'] == self.message_type and message['status'] == self.message_status
    return message['@type'] == self.message_type

  async def handle(self, message: dict):
    await self.message_handler(message)

  async def __call__(self, message: dict):
    await self.message_handler(message)


class Handlers():
  handlers: List[Handler]
  default_handler: Callable[[dict], Coroutine[Any, Any, None]]
  problem_report_handler: Callable[[dict], Coroutine[Any, Any, None]]

  def __init__(self):
    self.handlers = []

  def add_handler(
      self,
      message_type: str,
      message_status: int,
      message_handler: Callable[[dict], Coroutine[Any, Any, None]]):
    self.handlers.append(Handler(message_type, message_status, message_handler))

  def add_default_handler(self, handler: Callable[[dict], Coroutine[Any, Any, None]]):
    self.default_handler = handler

  def add_problem_report_handler(self, handler: Callable[[dict], Coroutine[Any, Any, None]]):
    self.problem_report_handler = handler

  async def handle_message(self, context: Context, raw_message: bytes):
    message: dict = await unpack_message(context, raw_message)
    handled: bool = False

    for handler in self.handlers:
      if handler.handles(message):
        await handler(message)
        handled = True

    if not handled:
      if is_problem_report(message['@type']) and self.problem_report_handler is not None:
        await self.problem_report_handler(message)
      elif self.default_handler is not None:
        await self.default_handler(message)


# Enables handler registration decorator @AddHandler(handlers, message_type, message_status)
class AddHandler():

  def __init__(self, handlers: Handlers, message_type: str, message_status: int = None):
    self.handlers = handlers
    self.message_type = message_type
    self.message_status = message_status

  def __call__(self, handler: Callable[[dict], Coroutine[Any, Any, None]], *args, **kwargs):
    self.handlers.add_handler(self.message_type, self.message_status, handler)



import json
import pytest

from verity_sdk.protocols.IssueCredential import IssueCredential
from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context
from test.test_utils import get_test_config, send_stub, cleanup

for_relationship = 'some_did'
name = 'degree'
cred_def_id = '12345'
credential_values = {
  'name': 'Joe',
  'degree': 'Bachelors',
  'gpa': '3.67'
}
price = '5'


def test_init():
  issueCredential = IssueCredential(for_relationship, name, cred_def_id, credential_values, price)

  assert issueCredential.for_relationship == for_relationship
  assert issueCredential.name == name
  assert issueCredential.cred_def_id == cred_def_id
  assert json.dumps(issueCredential.credential_values) == json.dumps(credential_values)


@pytest.mark.asyncio
async def test_offer_credential():
  context = await Context.create(await get_test_config())
  issueCredential = IssueCredential(for_relationship, name, cred_def_id, credential_values, price)
  issueCredential.send = send_stub
  msg = await issueCredential.offer_credential(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    IssueCredential.MSG_FAMILY,
    IssueCredential.MSG_FAMILY_VERSION,
    IssueCredential.OFFER_CREDENTIAL
  )
  assert msg['@id'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None
  assert msg['name'] == name
  assert msg['credDefId'] == cred_def_id
  assert msg['credentialValues']['name'] == credential_values['name']
  assert msg['credentialValues']['degree'] == credential_values['degree']
  assert msg['credentialValues']['gpa'] == credential_values['gpa']
  assert msg['price'] == price

  await cleanup(context)

@pytest.mark.asyncio
async def test_issue_credential():
  context = await Context.create(await get_test_config())
  issueCredential = IssueCredential(for_relationship, name, cred_def_id, credential_values, price)
  issueCredential.send = send_stub
  msg = await issueCredential.issue_credential(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    IssueCredential.MSG_FAMILY,
    IssueCredential.MSG_FAMILY_VERSION,
    IssueCredential.ISSUE_CREDENTIAL
  )
  assert msg['@id'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None

  await cleanup(context)

@pytest.mark.asyncio
async def test_status():
  context = await Context.create(await get_test_config())
  issueCredential = IssueCredential(for_relationship, name, cred_def_id, credential_values, price)
  issueCredential.send = send_stub
  msg = await issueCredential.status(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    IssueCredential.MSG_FAMILY,
    IssueCredential.MSG_FAMILY_VERSION,
    IssueCredential.GET_STATUS
  )
  assert msg['@id'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None

  await cleanup(context)



import abc

from verity_sdk.utils import Context, pack_message_for_verity, uuid
from verity_sdk.transports import send_message


class Protocol:
  MSG_FAMILY = 'none'
  MSG_FAMILY_VERSION = '0.0.0'

  class STATUS():
    pass

  messages: dict
  thread_id: str = uuid()

  @staticmethod
  async def get_message(context: Context, message: dict) -> bytes:
    return await pack_message_for_verity(context, message)

  @staticmethod
  async def send(context: Context, message: dict) -> bytes:
    message = await Protocol.get_message(context, message)
    send_message(context.verity_url, message)
    return message

  @staticmethod
  def get_new_id():
    return uuid()

  @abc.abstractmethod
  def define_messages(self):
    pass

  def get_thread_block(self) -> dict:
    return {
      'thid': self.thread_id
    }



import pytest
import requests

from verity_sdk.transports import send_message

def test_send_message_passes():
  send_message('https://jsonplaceholder.typicode.com/posts', b'Hello, world!')

def test_send_message_fails():
  with pytest.raises(requests.ConnectionError):
    assert not send_message('https://247530298573290582dfgsdsgsdfg.com/posts', b'Hello, world!')
  


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="verity-sdk",
    version="0.0.7",
    author="Evernym, Inc.",
    author_email="dev@evernym.com",
    description="The official Python SDK for Evernym's Verity",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
)




from verity_sdk.protocols.Protocol import Protocol
from verity_sdk.utils import get_message_type, get_problem_report_message_type, get_status_message_type
from verity_sdk.utils.Context import Context


class UpdateEndpoint(Protocol):
  MSG_FAMILY = 'configs'
  MSG_FAMILY_VERSION = '0.6'

  # Messages
  UPDATE_ENDPOINT = 'UPDATE_COM_METHOD'

  context: Context

  def __init__(self, context: Context):
    self.context = context
    self.define_messages()

  def define_messages(self):
    COM_METHOD_TYPE = 2

    self.messages = {
      self.UPDATE_ENDPOINT: {
        '@type': UpdateEndpoint.get_message_type(self.UPDATE_ENDPOINT),
        '@id': UpdateEndpoint.get_new_id(),
        'comMethod': {
          'id': 'webhook',
          'type': COM_METHOD_TYPE,
          'value': self.context.endpoint_url
        }
      }
    }

  @staticmethod
  def get_message_type(msg_name: str) -> str:
    return get_message_type(UpdateEndpoint.MSG_FAMILY, UpdateEndpoint.MSG_FAMILY_VERSION, msg_name)

  @staticmethod
  def get_problem_report_message_type() -> str:
    return get_problem_report_message_type(UpdateEndpoint.MSG_FAMILY, UpdateEndpoint.MSG_FAMILY_VERSION)

  @staticmethod
  def get_status_message_type() -> str:
    return get_status_message_type(UpdateEndpoint.MSG_FAMILY, UpdateEndpoint.MSG_FAMILY_VERSION)

  async def update(self) -> bytes:
    return await self.send(self.context, self.messages[self.UPDATE_ENDPOINT])



import json
from typing import Dict
from uuid import uuid4
from indy import crypto

from verity_sdk.utils import Context

MESSAGE_TYPE_DID = 'did:sov:123456789abcdefghi1234'


def prepare_forward_message(did: str, message: bytes) -> str:
  return json.dumps({
    '@type': 'did:sov:123456789abcdefghi1234;spec/routing/1.0/FWD',
    '@fwd': did,
    '@msg': json.loads(message.decode('utf-8'))
  })


async def pack_message_for_verity(context: Context, message: dict) -> bytes:
  agent_message = await crypto.pack_message(
    context.wallet_handle,
    json.dumps(message),
    [context.verity_pairwise_verkey],
    context.sdk_pairwise_verkey,
  )
  forward_message = prepare_forward_message(
    context.verity_pairwise_did,
    agent_message
  )
  return await crypto.pack_message(
    context.wallet_handle,
    forward_message,
    [context.verity_public_verkey],
    None
  )


async def unpack_forward_message(context: Context, message: bytes) -> Dict:
  unpacked_once_message = await unpack_message(context, message)
  return await unpack_message(
    context,
    json.dumps(unpacked_once_message['@msg']).encode('utf-8')
  )

def truncate_invite_details(invite_details: dict):
  truncated_invite_details = {
    'sc': invite_details['statusCode'],
    'id': invite_details['connReqId'],
    'sm': invite_details['statusMsg'],
    't': invite_details['targetName'],
    'version': invite_details['version'],
    's': {
      'n': invite_details['senderDetail']['name'],
      'd': invite_details['senderDetail']['DID'],
      'l': invite_details['senderDetail']['logoUrl'],
      'v': invite_details['senderDetail']['verKey'],
      'dp': {
        'd': invite_details['senderDetail']['agentKeyDlgProof']['agentDID'],
        'k': invite_details['senderDetail']['agentKeyDlgProof']['agentDelegatedKey'],
        's': invite_details['senderDetail']['agentKeyDlgProof']['signature']
      },
    },
    'sa': {
      'd': invite_details['senderAgencyDetail']['DID'],
      'e': invite_details['senderAgencyDetail']['endpoint'],
      'v': invite_details['senderAgencyDetail']['verKey']
    }
  }

  if 'publicDID' in invite_details['senderDetail']:
    truncated_invite_details['s']['publicDID'] = invite_details['senderDetail']['publicDID']

async def unpack_message(context: Context, message: bytes) -> Dict:
  jwe: bytes = await crypto.unpack_message(
    context.wallet_handle,
    message
  )
  message = json.loads(jwe.decode('utf-8'))['message']
  return json.loads(message)


def uuid() -> str:
  return str(uuid4())


def get_message_type(msg_family: str, msg_family_version: str, msg_name: str) -> str:
  return '{};spec/{}/{}/{}'.format(MESSAGE_TYPE_DID, msg_family, msg_family_version, msg_name)


def get_problem_report_message_type(msg_family: str, msg_family_version: str) -> str:
  return get_message_type(msg_family, msg_family_version, 'problem-report')


def get_status_message_type(msg_family: str, msg_family_version: str) -> str:
  return get_message_type(msg_family, msg_family_version, 'status')



import pytest

from verity_sdk.protocols.QuestionAnswer import QuestionAnswer
from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context
from test.test_utils import get_test_config, send_stub, cleanup


for_relationship = 'some_did'
question_text = 'Are you trying to login to acme.com?'
question_detail = 'IP Address: 56.24.11.126'
valid_responses = ['Yes', 'No, that\'s not me!']
signature_required = True


def test_init():
  question_answer = QuestionAnswer(for_relationship, question_text, question_detail, valid_responses, signature_required)

  assert question_answer.for_relationship == for_relationship
  assert question_answer.question_text == question_text
  assert question_answer.question_detail == question_detail
  assert question_answer.valid_responses == valid_responses
  assert question_answer.signature_required == signature_required


@pytest.mark.asyncio
async def test_ask():
  context = await Context.create(await get_test_config())
  question_answer = QuestionAnswer(for_relationship, question_text, question_detail, valid_responses, signature_required)
  question_answer.send = send_stub
  msg = await question_answer.ask(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    QuestionAnswer.MSG_FAMILY,
    QuestionAnswer.MSG_FAMILY_VERSION,
    QuestionAnswer.ASK_QUESTION
  )
  assert msg['@id'] is not None
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['text'] == question_text
  assert msg['detail'] == question_detail
  assert msg['valid_responses'] == valid_responses
  assert msg['signature_required'] == signature_required

  await cleanup(context)

async def test_status():
  context = await Context.create(await get_test_config())
  question_answer = QuestionAnswer(for_relationship, question_text, question_detail, valid_responses, signature_required)
  question_answer.send = send_stub
  msg = await question_answer.status(context)
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/{}/{}/{}'.format(
    MESSAGE_TYPE_DID,
    QuestionAnswer.MSG_FAMILY,
    QuestionAnswer.MSG_FAMILY_VERSION,
    QuestionAnswer.GET_STATUS
  )
  assert msg['@id'] is not None
  assert msg['~for_relationship'] == for_relationship
  assert msg['~thread'] is not None
  assert msg['~thread']['thid'] is not None

  await cleanup(context)



import pytest

from test.test_utils import get_test_config, send_stub, cleanup
from verity_sdk.protocols.UpdateEndpoint import UpdateEndpoint
from verity_sdk.utils import unpack_forward_message, MESSAGE_TYPE_DID
from verity_sdk.utils.Context import Context

@pytest.mark.asyncio
async def test_init():
  context = await Context.create(await get_test_config())
  update_endpoint = UpdateEndpoint(context)
  assert update_endpoint.context.endpoint_url == context.endpoint_url
  await cleanup(context)

@pytest.mark.asyncio
async def test_write():
  context = await Context.create(await get_test_config())
  update_endpoint = UpdateEndpoint(context)
  update_endpoint.send = send_stub
  msg = await update_endpoint.update()
  msg = await unpack_forward_message(context, msg)

  assert msg['@type'] == '{};spec/configs/0.6/UPDATE_COM_METHOD'.format(MESSAGE_TYPE_DID)
  assert msg['@id'] is not None
  assert msg['comMethod']['id'] == 'webhook'
  assert msg['comMethod']['type'] == 2
  assert msg['comMethod']['value'] == context.endpoint_url

  await cleanup(context)

def test_get_message_type():
  assert UpdateEndpoint.get_message_type('message_name') == '{};spec/configs/0.6/message_name'.format(MESSAGE_TYPE_DID)


def test_get_problem_report_message_type():
  assert UpdateEndpoint.get_problem_report_message_type() == '{};spec/configs/0.6/problem-report'.format(MESSAGE_TYPE_DID)


def test_get_status_message_type():
  assert UpdateEndpoint.get_status_message_type() == '{};spec/configs/0.6/status'.format(MESSAGE_TYPE_DID)
